// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Graphics.VS.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__GRAPHICS__GRAPHICS_VS
#define TINMAN__GRAPHICS__GRAPHICS_VS

#include "../../GpuX.hlsl"
#include "../Configuration.hlsl"
#include "../Util.hlsl"
#include "CB_Graphics_Dynamic.hlsl"
#include "CB_Graphics_Static.hlsl"
#include "CB_Graphics_Volatile.hlsl"
#include "Graphics.hlsl"
#include "VS_In.hlsl"
#include "VS_Out.hlsl"

// -------------------------------------------------------------------------------------------------
// Vertex Shader 'Graphics.VS'
// -------------------------------------------------------------------------------------------------

VS_Out main(VS_In data)
{
  VS_Out result;

  result.Screen = mul(g_screenMatrix, data.Position.xy) + g_screenOffset;
  result.Position = float4(2.0 * (result.Screen.x - PIXEL_OFFSET) / g_screenSize.x - 1.0, 1.0 - 2.0
      * (result.Screen.y - PIXEL_OFFSET) / g_screenSize.y, data.Position.z, 1.0);
  result.Color = FromSrgb(data.Color) * g_color;
  result.Coords = data.Coords;
  result.Clip = 0.0;

  return result;
}

#endif
