// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Mipmap.CS_Linear.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__MIPMAP__MIPMAP_CS_LINEAR
#define TINMAN__MIPMAP__MIPMAP_CS_LINEAR

#include "../../GpuX.hlsl"
#include "CB_Mipmap_Static.hlsl"
#include "Mipmap.hlsl"
#include "SR_Mipmap.hlsl"

// -------------------------------------------------------------------------------------------------
// Compute Shader 'Mipmap.CS_Linear'
// -------------------------------------------------------------------------------------------------

[numthreads(32,32,1)]
void main(int3 threadId : SV_GroupThreadID)
{
  for (int level = 0; level < g_levels - 1; level++)
  {
    int2 size;

    gxSizeOf(g_texture[level], size);

    for (int y1 = threadId.y * 2; y1 < size.y; y1 += 64)
    {
      int y2;

      y2 = min(y1 + 1, size.y - 1);

      for (int x1 = threadId.x * 2; x1 < size.x; x1 += 64)
      {
        float4 s;
        int x2;

        x2 = min(x1 + 1, size.x - 1);
        s = gxTexture2DGet(g_texture[level], int2(x1, y1));
        s += gxTexture2DGet(g_texture[level], int2(x1, y2));
        s += gxTexture2DGet(g_texture[level], int2(x2, y1));
        s += gxTexture2DGet(g_texture[level], int2(x2, y2));

        gxTexture2DSet(g_texture[level + 1], int2(x1 / 2, y1 / 2), s / 4);
      }
    }

    gxSync();
  }
}

#endif
