// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Model.VS_Default.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__MODEL__MODEL_VS_DEFAULT
#define TINMAN__MODEL__MODEL_VS_DEFAULT

#include "../../GpuX.hlsl"
#include "../Camera.hlsl"
#include "../CB_Camera_Static.hlsl"
#include "../CB_Lighting_Static.hlsl"
#include "../Util.hlsl"
#include "CB_Model_Dynamic.hlsl"
#include "CB_Model_Volatile.hlsl"
#include "Model.hlsl"
#include "VS_In.hlsl"
#include "VS_Out.hlsl"

// -------------------------------------------------------------------------------------------------
// Vertex Shader 'Model.VS_Default'
// -------------------------------------------------------------------------------------------------

VS_Out main(VS_In vertex)
{
  VS_Out result;
  float3 position;

  // Apply model transformation.

  ModelTransform(vertex.Position, vertex.PositionExtra);

  position = ModelPosition(false, vertex.Position, vertex.PositionExtra);

  // Compute output values.

  VertexColorAlphaOcclusion(vertex.Color, result.Color, result.DistanceOcclusion.y);

  result.Coords = float4(vertex.Coords[0], vertex.Coords[1]);
  result.DistanceOcclusion.x = length(position);
  result.Eye = mul(g_lightSpace, position / -result.DistanceOcclusion.x);
  result.Plain.xyz = position;
  result.Position = mul(g_viewProjectionMatrix[0], float4(position, 1));
  result.Plain.w = result.Position.z;
  result.Shadow = ComputeDotDirection(true, ModelPosition(true, vertex.Position, vertex.
      PositionExtra));
  result.Tangent = g_tangentMask.x == 0.0 ? 0.0 : mul(g_lightSpace, mul(g_modelMatrixIT, float4(
      DecodeShift(vertex.Tangent), 0)));
  result.Bitangent = g_tangentMask.y == 0.0 ? 0.0 : mul(g_lightSpace, mul(g_modelMatrixIT, float4(
      DecodeShift(vertex.Bitangent), 0)));
  result.Normal = g_tangentMask.z == 0.0 ? 0.0 : mul(g_lightSpace, mul(g_modelMatrixIT, float4(
      DecodeShift(vertex.Normal), 0)));
  result.Clip = 0.0;

  return result;
}

#endif
