// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Process.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__PROCESS__PROCESS
#define TINMAN__PROCESS__PROCESS

#include "../../GpuX.hlsl"
#include "CB_Process_Static.hlsl"
#include "SR_Process.hlsl"

// -------------------------------------------------------------------------------------------------
// Prototypes
// -------------------------------------------------------------------------------------------------

/*!
 * @brief Computes a 7x7 Gaussian blur for a single pass.
 *
 * See here for details:
 * https://software.intel.com/en-us/blogs/2014/07/15/an-investigation-of-fast-real-time-gpu-based-image-blur-algorithms
 *
 * @param coords The texture coordinates.
 *
 * @returns The blurred texture sample.
 */
float2 Blur(float2 coords);

// -------------------------------------------------------------------------------------------------
// Functions
// -------------------------------------------------------------------------------------------------

float2 Blur(float2 coords)
{
  float2 result;
  float2 ofs;

  ofs = g_offset * 0.53805;
  result = (gxTextures2D(g_input, 0, coords + ofs) + gxTextures2D(g_input, 0, coords - ofs)).rg *
      0.44908;
  ofs = g_offset * 2.06278;
  result += (gxTextures2D(g_input, 0, coords + ofs) + gxTextures2D(g_input, 0, coords - ofs)).rg *
      0.05092;

  return result;
}

#endif
