// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Renderer.GS.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__RENDERER__RENDERER_GS
#define TINMAN__RENDERER__RENDERER_GS

#include "../../GpuX.hlsl"
#include "CB_Renderer_Dynamic.hlsl"
#include "Renderer.hlsl"
#include "VS_Out.hlsl"

// -------------------------------------------------------------------------------------------------
// Geometry Shader 'Renderer.GS'
// -------------------------------------------------------------------------------------------------

[maxvertexcount(8)]
void main(line VS_Out data[2], inout TriangleStream<VS_Out> stream)
{
  float threshold;
  VS_Out v;
  float2 a;
  float2 b;
  float2 dx;
  float2 dy;
  float c;

  // Clip line segment at Z = ~0, to avoid logarithmic depth clipping distortions.

  threshold = max(data[0].LinearZ, data[1].LinearZ) / 65536.0;

  if (data[1].LinearZ < threshold)
  {
    float f;

    f = (threshold - data[0].LinearZ) / (data[1].LinearZ - data[0].LinearZ);
    data[1].Color *= f;
    data[1].Position *= f;
    f = 1.0 - f;
    data[1].Color += data[0].Color * f;
    data[1].Position += data[0].Position * f;
    data[1].LinearZ = threshold;
    data[1].Position.z = threshold * data[1].Position.w;
  }
  else if (data[0].LinearZ < threshold)
  {
    float f;

    f = (threshold - data[1].LinearZ) / (data[0].LinearZ - data[1].LinearZ);
    data[0].Color *= f;
    data[0].Position *= f;
    f = 1.0 - f;
    data[0].Color += data[1].Color * f;
    data[0].Position += data[1].Position * f;
    data[0].LinearZ = threshold;
    data[0].Position.z = threshold * data[0].Position.w;
  }

  // Extrude line segment to triangle strip in clip-space, then make use of texture coordinate
  // interpolation to generate round corners.
  // 0---1---2---3
  // |           |
  // |   a---b   |
  // |           |
  // 4---5---6---7

  data[0].Color.a = 1.0;
  data[1].Color.a = 1.0;
  a = data[0].Position.xy / data[0].Position.w;
  b = data[1].Position.xy / data[1].Position.w;
  dx = normalize(b - a);
  dy = float2(-dx.y, dx.x);
  dx *= g_lineThickness.xy;
  dy *= g_lineThickness.xy;
  c = g_lineThickness.z + 1.0;

  // 0, 4, 1, 5

  {
    float w;

    w = data[0].Position.w;
    v = data[0];
    v.Position.xy += (-dx - dy) * w;
    v.Clip = float2(-c, -c);

    gxStreamAppend(stream, v);

    v = data[0];
    v.Position.xy += (-dx + dy) * w;
    v.Clip = float2(-c, c);

    gxStreamAppend(stream, v);

    v = data[0];
    v.Position.xy += -dy * w;
    v.Clip = float2(0, -c);

    gxStreamAppend(stream, v);

    v = data[0];
    v.Position.xy += dy * w;
    v.Clip = float2(0, c);

    gxStreamAppend(stream, v);
  }

  // 2, 6, 3, 7

  {
    float w;

    w = data[1].Position.w;
    v = data[1];
    v.Position.xy += -dy * w;
    v.Clip = float2(0, -c);

    gxStreamAppend(stream, v);

    v = data[1];
    v.Position.xy += dy * w;
    v.Clip = float2(0, c);

    gxStreamAppend(stream, v);

    v = data[1];
    v.Position.xy += (dx - dy) * w;
    v.Clip = float2(c, -c);

    gxStreamAppend(stream, v);

    v = data[1];
    v.Position.xy += (dx + dy) * w;
    v.Clip = float2(c, c);

    gxStreamAppend(stream, v);
  }
}

#endif
