// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       DisplacementFixed.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__DISPLACEMENTFIXED
#define TINMAN__TERRAIN__DISPLACEMENTFIXED

#include "../../GpuX.hlsl"

// -------------------------------------------------------------------------------------------------
// Shader Parameters
// -------------------------------------------------------------------------------------------------

struct DisplacementFixed
{
  /*!
   * @brief The material IDs per slot.
   */
  int4 MaterialIds SEMANTIC_CUSTOM(DISPLACEMENTFIXED_MATERIALIDS);

  /*!
   * @brief Unit-length eye-vector in light-space.
   */
  float3 Eye SEMANTIC_CUSTOM(DISPLACEMENTFIXED_EYE);

  /*!
   * @brief Material texture-coordinates in light-space.
   */
  float3 MaterialCoords SEMANTIC_CUSTOM(DISPLACEMENTFIXED_MATERIALCOORDS);

  /*!
   * @brief Material texture-coordinates in light-space.
   */
  float3 MaterialNormal SEMANTIC_CUSTOM(DISPLACEMENTFIXED_MATERIALNORMAL);

  /*!
   * @brief Unit-length normal vector in light-space.
   */
  float3 Normal SEMANTIC_CUSTOM(DISPLACEMENTFIXED_NORMAL);

  /*!
   * @brief Original vertex position, in camera-space.
   */
  float3 Position SEMANTIC_CUSTOM(DISPLACEMENTFIXED_POSITION);

  /*!
   * @brief Original vertex position, in locked camera-space.
   */
  float3 Shadow SEMANTIC_CUSTOM(DISPLACEMENTFIXED_SHADOW);

  /*!
   * @brief Area of vertex neighborhood.
   */
  float Area SEMANTIC_CUSTOM(DISPLACEMENTFIXED_AREA);

  /*!
   * @brief Distance from vertex to camera.
   */
  float Distance SEMANTIC_CUSTOM(DISPLACEMENTFIXED_DISTANCE);
};

#endif
