// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Terrain.GS_Material.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TERRAIN_GS_MATERIAL
#define TINMAN__TERRAIN__TERRAIN_GS_MATERIAL

#include "../../GpuX.hlsl"
#include "DisplacementVarying.hlsl"
#include "GS_In_Material.hlsl"
#include "PS_In_Material.hlsl"
#include "Terrain.hlsl"

// -------------------------------------------------------------------------------------------------
// Geometry Shader 'Terrain.GS_Material'
// -------------------------------------------------------------------------------------------------

[maxvertexcount(24)]
void main(triangle GS_In_Material data[3], inout TriangleStream<PS_In_Material> stream)
{
  DisplacementVarying displacement[3];
  GS_In_Material input0;
  GS_In_Material input1;
  GS_In_Material input2;
  DisplacementVarying displacement0;
  DisplacementVarying displacement1;
  DisplacementVarying displacement2;
  PS_In_Material v[15];
  float3 f;
  float fMin;
  float fMax;

  // Compute tessellation factors for each edge:
  // f[0] : v0 <-> v1
  // f[1] : v1 <-> v2
  // f[2] : v2 <-> v0

  displacement[0] = ComputeDisplacementVarying(data[0].Fixed, data[0].WeightsA, true);
  displacement[1] = ComputeDisplacementVarying(data[1].Fixed, data[1].WeightsA, true);
  displacement[2] = ComputeDisplacementVarying(data[2].Fixed, data[2].WeightsA, true);
  f.x = MaterialTessellationFactor(data[0], data[1], displacement[0], displacement[1]);
  f.y = MaterialTessellationFactor(data[1], data[2], displacement[1], displacement[2]);
  f.z = MaterialTessellationFactor(data[2], data[0], displacement[2], displacement[0]);
  f = min(2.0, floor(f / 2.0));
  fMin = min(f.x, min(f.y, f.z));
  fMax = max(f.x, max(f.y, f.z));

  // ID  |  0 |  1 |  2 |  3 |  4 |  5 |  6 |  7 |  8 |  9 | 10 | 11 | 12 | 13 | 14 |
  // ----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+
  // 000 |  X |  X |  X |    |    |    |    |    |    |    |    |    |    |    |    |
  // 001 |  X |  X |  X |    |    |  X |    |    |    |    |    |    |    |    |    |
  // 002 |  X |  X |  X |    |    |  X |    |    |    |    |  X |  X |    |    |    |
  // 010 |  X |  X |  X |    |  X |    |    |    |    |    |    |    |    |    |    |
  // 011 |  X |  X |  X |    |  X |  X |    |    |    |    |    |    |    |    |    |
  // 012 |  X |  X |  X |    |  X |  X |    |    |    |    |  X |  X |    |    |    |
  // 020 |  X |  X |  X |    |  X |    |    |    |  X |  X |    |    |    |    |    |
  // 021 |  X |  X |  X |    |  X |  X |    |    |  X |  X |    |    |    |    |    |
  // 022 |  X |  X |  X |    |  X |  X |    |    |  X |  X |  X |  X |    |    |    |
  // 100 |  X |  X |  X |  X |    |    |    |    |    |    |    |    |    |    |    |
  // 101 |  X |  X |  X |  X |    |  X |    |    |    |    |    |    |    |    |    |
  // 102 |  X |  X |  X |  X |    |  X |    |    |    |    |  X |  X |    |    |    |
  // 110 |  X |  X |  X |  X |  X |    |    |    |    |    |    |    |    |    |    |
  // 111 |  X |  X |  X |  X |  X |  X |    |    |    |    |    |    |    |    |    |
  // 112 |  X |  X |  X |  X |  X |  X |    |    |    |    |  X |  X |    |    |    |
  // 120 |  X |  X |  X |  X |  X |    |    |    |  X |  X |    |    |    |    |    |
  // 121 |  X |  X |  X |  X |  X |  X |    |    |  X |  X |    |    |    |    |    |
  // 122 |  X |  X |  X |  X |  X |  X |    |    |  X |  X |  X |  X |    |    |    |
  // 200 |  X |  X |  X |  X |    |    |  X |  X |    |    |    |    |    |    |    |
  // 201 |  X |  X |  X |  X |    |  X |  X |  X |    |    |    |    |    |    |    |
  // 202 |  X |  X |  X |  X |    |  X |  X |  X |    |    |  X |  X |    |    |    |
  // 210 |  X |  X |  X |  X |  X |    |  X |  X |    |    |    |    |    |    |    |
  // 211 |  X |  X |  X |  X |  X |  X |  X |  X |    |    |    |    |    |    |    |
  // 212 |  X |  X |  X |  X |  X |  X |  X |  X |    |    |  X |  X |    |    |    |
  // 220 |  X |  X |  X |  X |  X |    |  X |  X |  X |  X |    |    |    |    |    |
  // 221 |  X |  X |  X |  X |  X |  X |  X |  X |  X |  X |    |    |    |    |    |
  // 222 |  X |  X |  X |  X |  X |  X |  X |  X |  X |  X |  X |  X |  X |  X |  X |
  // Compute base vertices after rotating them of necessary, in order reduce permutations.

  switch ((int) dot(f, float3(9, 3, 1)))
  {
    case 1:
      // 001

    case 2:
      // 002

    case 10:
      // 101

    case 11:
      // 102

    case 14:
      // 112

    case 19:
      // 201

    case 20:
      // 202

    case 23:
      // 212
      // Rotate right.

      f = f.zxy;
      input0 = data[2];
      input1 = data[0];
      input2 = data[1];
      displacement0 = displacement[2];
      displacement1 = displacement[0];
      displacement2 = displacement[1];

      break;

    case 3:
      // 010

    case 4:
      // 011

    case 5:
      // 012

    case 6:
      // 020

    case 7:
      // 021

    case 8:
      // 022

    case 16:
      // 121

    case 17:
      // 122
      // Rotate left.

      f = f.yzx;
      input0 = data[1];
      input1 = data[2];
      input2 = data[0];
      displacement0 = displacement[1];
      displacement1 = displacement[2];
      displacement2 = displacement[0];

      break;

    default:
      input0 = data[0];
      input1 = data[1];
      input2 = data[2];
      displacement0 = displacement[0];
      displacement1 = displacement[1];
      displacement2 = displacement[2];

      break;
  }

  v[0] = MaterialCompute(input0, displacement0);
  v[1] = MaterialCompute(input1, displacement1);
  v[2] = MaterialCompute(input2, displacement2);

  // Early exit for trivial case.

  if (fMax == 0.0)
  {
    // 0
    // |\
    // | \
    // |  \
    // |   \
    // |    \
    // |     \
    // |      \
    // |       \
    // |        \
    // |         \
    // |          \
    // |           \
    // |            \
    // |             \
    // |              \
    // 1---------------2

    gxStreamAppend(stream, v[0]);
    gxStreamAppend(stream, v[1]);
    gxStreamAppend(stream, v[2]);
  }
  else
  {
    GS_In_Material input3;
    GS_In_Material input4;
    GS_In_Material input5;

    // Compute vertex data for required tessellation vertices.

    input3 = MaterialMix2(input0, input1);
    input4 = MaterialMix2(input1, input2);
    input5 = MaterialMix2(input2, input0);

    if (f.x > 0.0)
    {
      v[3] = MaterialCompute(input3);

      if (f.x > 1.0)
      {
        v[6] = MaterialCompute(MaterialMix2(input0, input3));
        v[7] = MaterialCompute(MaterialMix2(input3, input1));
      }
      else
        v[6] = v[7] = v[0];
    }
    else
      v[3] = v[6] = v[7] = v[0];

    if (f.y > 0.0)
    {
      v[4] = MaterialCompute(input4);

      if (f.y > 1.0)
      {
        v[8] = MaterialCompute(MaterialMix2(input1, input4));
        v[9] = MaterialCompute(MaterialMix2(input4, input2));
      }
      else
        v[8] = v[9] = v[0];
    }
    else
      v[4] = v[8] = v[9] = v[0];

    if (f.z > 0.0)
    {
      v[5] = MaterialCompute(input5);

      if (f.z > 1.0)
      {
        v[10] = MaterialCompute(MaterialMix2(input2, input5));
        v[11] = MaterialCompute(MaterialMix2(input5, input0));
      }
      else
        v[10] = v[11] = v[0];
    }
    else
      v[5] = v[10] = v[11] = v[0];

    if (fMin > 1.0)
    {
      v[12] = MaterialCompute(MaterialMix2(input5, input3));
      v[13] = MaterialCompute(MaterialMix2(input3, input4));
      v[14] = MaterialCompute(MaterialMix2(input4, input5));
    }
    else
      v[12] = v[13] = v[14] = v[0];

    // Output triangulation for tessellated triangle.
    // 0                  0
    // |\                 |\
    // | \                | \
    // |  \               |  \
    // |   \              6--11
    // |    \             |\  |\
    // |     \            | \ | \
    // |      \           |  \|  \
    // 3-------5          3--12---5
    // |\      |\         |\  |\  |\
    // | \     | \        | \ | \ | \
    // |  \    |  \       |  \|  \|  \
    // |   \   |   \      7--13--14--10
    // |    \  |    \     |\  |\  |\  |\
    // |     \ |     \    | \ | \ | \ | \
    // |      \|      \   |  \|  \|  \|  \
    // 1-------4-------2  1---8---4---9---2

    switch ((int) dot(f, float3(9, 3, 1)))
    {
      case 9:
        // 100

        gxStreamAppend(stream, v[0]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[2]);
        gxStreamAppend(stream, v[1]);

        break;

      case 12:
        // 110

        gxStreamAppend(stream, v[2]);
        gxStreamAppend(stream, v[0]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[1]);

        break;

      case 13:
        // 111

        gxStreamAppend(stream, v[0]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[5]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[2]);
        gxStreamReset(stream);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[1]);
        gxStreamAppend(stream, v[4]);

        break;

      case 15:
        // 120

        gxStreamAppend(stream, v[1]);
        gxStreamAppend(stream, v[8]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[0]);
        gxStreamAppend(stream, v[9]);
        gxStreamAppend(stream, v[2]);

        break;

      case 18:
        // 200

        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[2]);
        gxStreamAppend(stream, v[6]);
        gxStreamAppend(stream, v[0]);
        gxStreamReset(stream);
        gxStreamAppend(stream, v[1]);
        gxStreamAppend(stream, v[2]);
        gxStreamAppend(stream, v[7]);
        gxStreamAppend(stream, v[3]);

        break;

      case 21:
        // 210

        gxStreamAppend(stream, v[0]);
        gxStreamAppend(stream, v[6]);
        gxStreamAppend(stream, v[2]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[7]);
        gxStreamAppend(stream, v[1]);

        break;

      case 22:
        // 211

        gxStreamAppend(stream, v[0]);
        gxStreamAppend(stream, v[6]);
        gxStreamAppend(stream, v[5]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[7]);
        gxStreamAppend(stream, v[1]);
        gxStreamReset(stream);
        gxStreamAppend(stream, v[5]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[2]);

        break;

      case 24:
        // 220

        gxStreamAppend(stream, v[1]);
        gxStreamAppend(stream, v[8]);
        gxStreamAppend(stream, v[7]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[9]);
        gxStreamAppend(stream, v[6]);
        gxStreamAppend(stream, v[2]);
        gxStreamAppend(stream, v[0]);

        break;

      case 25:
        // 221

        gxStreamAppend(stream, v[0]);
        gxStreamAppend(stream, v[6]);
        gxStreamAppend(stream, v[5]);
        gxStreamAppend(stream, v[9]);
        gxStreamAppend(stream, v[2]);
        gxStreamReset(stream);
        gxStreamAppend(stream, v[1]);
        gxStreamAppend(stream, v[8]);
        gxStreamAppend(stream, v[7]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[9]);
        gxStreamAppend(stream, v[6]);

        break;

      default:
        // 222

        gxStreamAppend(stream, v[0]);
        gxStreamAppend(stream, v[6]);
        gxStreamAppend(stream, v[11]);
        gxStreamAppend(stream, v[12]);
        gxStreamAppend(stream, v[5]);
        gxStreamAppend(stream, v[14]);
        gxStreamAppend(stream, v[10]);
        gxStreamAppend(stream, v[9]);
        gxStreamAppend(stream, v[2]);
        gxStreamReset(stream);
        gxStreamAppend(stream, v[6]);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[12]);
        gxStreamAppend(stream, v[13]);
        gxStreamAppend(stream, v[14]);
        gxStreamAppend(stream, v[4]);
        gxStreamAppend(stream, v[9]);
        gxStreamReset(stream);
        gxStreamAppend(stream, v[3]);
        gxStreamAppend(stream, v[7]);
        gxStreamAppend(stream, v[13]);
        gxStreamAppend(stream, v[8]);
        gxStreamAppend(stream, v[4]);
        gxStreamReset(stream);
        gxStreamAppend(stream, v[7]);
        gxStreamAppend(stream, v[1]);
        gxStreamAppend(stream, v[8]);

        break;
    }
  }
}

#endif
