// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Terrain.VS_Texture.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TERRAIN_VS_TEXTURE
#define TINMAN__TERRAIN__TERRAIN_VS_TEXTURE

#include "../../GpuX.hlsl"
#include "../Camera.hlsl"
#include "../CB_Lighting_Static.hlsl"
#include "../Util.hlsl"
#include "PS_In_Texture.hlsl"
#include "SR_Terrain.hlsl"
#include "Terrain.hlsl"
#include "VS_In.hlsl"

// -------------------------------------------------------------------------------------------------
// Vertex Shader 'Terrain.VS_Texture'
// -------------------------------------------------------------------------------------------------

PS_In_Texture main(VS_In data)
{
  PS_In_Texture result;
  float3 position;
  float distanceSqr;
  float distance;

  data.Normal = DecodeShift(data.Normal);
  data.NormalBase = DecodeShift(data.NormalBase);

  // Compute shadow depth.

  result.Shadow = ComputeDotDirection(true, ComputePosition(true, data.Position, data.PositionDelta)
      );

  // Compute vertex position.

  position = ComputePosition(false, data.Position, data.PositionDelta);
  result.Position = ProjectPosition(position);
  distanceSqr = dot(position, position);
  distance = sqrt(distanceSqr);
  result.Normal = normalize(mul(g_lightSpace, lerp(data.NormalBase, data.Normal, VertexBlend(data.
      LevelOfDetail, distanceSqr))));
  result.EyeDistance.xyz = mul(g_lightSpace, -position / distance);
  result.Color = FromSrgb(data.Color);
  result.EyeDistance.w = sqrt(dot(result.Shadow.xyz, result.Shadow.xyz));

  for (int i = 0; i < TextureCount; i++)
  {
    float2 xy;
    float2 zw;

    TextureCoords(i, data.CoordsHigh, data.CoordsLow, xy, zw);

    result.TexCoords[i] = float4(xy, zw);
  }

  return result;
}

#endif
